/*
 * Copyright 2011-13 Fraunhofer ISE 
 *    
 * This file is part of openMUC.
 * For more information visit http://www.openmuc.org
 * 
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with openMUC.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package org.openmuc.framework.driver.ehz;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.openmuc.framework.config.ChannelScanInformation;
import org.openmuc.framework.data.ValueType;
import org.openmuc.framework.driver.ehz.iec62056_21.DataSet;
import org.openmuc.framework.driver.ehz.iec62056_21.IecReceiver;
import org.openmuc.framework.driver.ehz.iec62056_21.ModeDMessage;
import org.openmuc.framework.driver.spi.ChannelRecordContainer;
import org.openmuc.framework.driver.spi.ConnectionException;

/**
 * @author Frederic Robra
 *
 */
public class IecConnection extends Connection {

	private IecReceiver receiver;
	
	
	public IecConnection(String deviceAddress, int timeout) throws ConnectionException {
		try {
			receiver = new IecReceiver(deviceAddress);
		} catch (Exception e) {
			throw new ConnectionException("serial port not found");
		}
	}
	
	
	/* (non-Javadoc)
	 * @see org.openmuc.framework.driver.ehz.Connection#close()
	 */
	@Override
	public void close() {
		receiver.close();
	}

	/* (non-Javadoc)
	 * @see org.openmuc.framework.driver.ehz.Connection#read(java.util.List, int)
	 */
	@Override
	public void read(List<ChannelRecordContainer> containers, int timeout) {
		logger.trace("iec read");
		try {
			long timestamp = System.currentTimeMillis();
			
			byte[] frame = receiver.receiveMessage(timeout);
			ModeDMessage message = new ModeDMessage(frame);
			message.parse();
			List<String> dataSets = message.getDataSets();
			
			Map<String, Double> values = new LinkedHashMap<String, Double>();
			for (String data : dataSets) {
				DataSet dataSet = new DataSet(data);
				String address = dataSet.getAddress();
				double value = dataSet.getVal();
				values.put(address, value);
				logger.trace("sml found " + value + "@" + address);
			}
			
			handleChannelRecordContainer(containers, values, timestamp);
		}
		catch(Exception e) {
			logger.error("sml read failed");
		}
	}


	/* (non-Javadoc)
	 * @see org.openmuc.framework.driver.ehz.Connection#listChannels(int)
	 */
	@Override
	public List<ChannelScanInformation> listChannels(int timeout) {
		List<ChannelScanInformation> channelInfos = new LinkedList<ChannelScanInformation>();
		
		logger.debug("scanning channels");
		try {
			byte[] frame = receiver.receiveMessage(timeout);
			ModeDMessage message = new ModeDMessage(frame);
			message.parse();
			List<String> dataSets = message.getDataSets();
			
			for (String data : dataSets) {
				DataSet dataSet = new DataSet(data);
				String channelAddress = dataSet.getAddress();
				String description = "Current value: " + dataSet.getVal() + dataSet.getUnit();
				ValueType valueType = ValueType.DOUBLE;
				Integer valueTypeLength = null;
				Boolean readable = true;
				Boolean writable = false;
				ChannelScanInformation channelInfo = new ChannelScanInformation(channelAddress, description, valueType,
						valueTypeLength, readable, writable);
				channelInfos.add(channelInfo);
			}
			
		}
		catch(Exception e) {
			e.printStackTrace();
			logger.warn("read failed");
		}
		return channelInfos;
	}


	/* (non-Javadoc)
	 * @see org.openmuc.framework.driver.ehz.Connection#isWorking()
	 */
	@Override
	public boolean isWorking() {
		try {
			byte[] frame = receiver.receiveMessage(1000);
			ModeDMessage message = new ModeDMessage(frame);
			message.parse();
			return true;
		} catch (Exception e) {
			return false;
		}
		
	}

}
